﻿Type=Class
Version=4.7
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Region Notes
' Server WebSocket Class Module: ESPDashboardPage
' Purpose:
' Manage browser connection
#End Region

'
Sub Class_Globals
	Private ws As WebSocket 'ignore
	' will hold our page information
	Public page As ABMPage
	' page theme
	Private theme As ABMTheme
	' to access the constants
	Private ABM As ABMaterial 'ignore	
	' name of the page, must be the same as the class name (case sensitive!)
	Public Name As String = "ESPDashboardPage"
	' name of the app, same as in ABMApplication
	Public AppName As String = "ESPDashboard"
	
	' own project vars
	Dim CurrentValuesT As List
	Dim CurrentLabelsT As List
	Dim CurrentValuesP As List
	Dim CurrentLabelsP As List
	Dim DataCollectionTimer As Timer
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	' build the local structure IMPORTANT!
	DateTime.TimeFormat = "HH:mm"
	'
	CurrentLabelsT.Initialize
	CurrentValuesT.Initialize
	CurrentLabelsP.Initialize
	CurrentValuesP.Initialize
	'
	BuildPage
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	Log("WebSocket Connected")
	ws = WebSocket1	
	If ABMShared.NeedsAuthorization Then
		If ws.Session.GetAttribute2("IsAuthorized", "") = "" Then
			ABMShared.NavigateToPage(ws, "../")
			Return
		End If
	End If
	' connect our page with the websocket	
	page.SetWebSocket(ws)	
	' Prepare the page IMPORTANT!
	page.Prepare	
	DataCollectionTimer.Initialize("DataCollectionTimer", 5000)
	DataCollectionTimer.Enabled = True
End Sub

Private Sub WebSocket_Disconnected
	DataCollectionTimer.Enabled = False
	Log("WebSocket Disconnected")
End Sub

Sub Page_ParseEvent(Params As Map) 
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	If SubExists(Me, eventName) Then
		Params.Remove("eventname")
		Params.Remove("eventparams")
		Select Case Params.Size
			Case 0
				CallSub(Me, eventName)
			Case 1
				CallSub2(Me, eventName, Params.Get(eventParams(0)))					
			Case 2
				If Params.get(eventParams(0)) = "abmistable" Then
					Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
					CallSub2(Me, eventName, PassedTables)
				Else
					CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
				End If
			Case Else
				' cannot be called diretly, to many param
				CallSub2(Me, eventName, Params)				
		End Select
	End If
End Sub

public Sub BuildTheme()
	' start with the base theme defined in ABMShared
	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)
	
	' add additional themes specific for this page
End Sub

public Sub BuildPage()
	' initialize the theme
	BuildTheme
	
	' initialize this page using our theme
	page.InitializeWithTheme(Name, "/ws/" & AppName & "/" & Name, False, 300, theme)
	page.ShowLoader=True
	page.PageHTMLName = "index.html"
	page.PageTitle = "B4J HowTo ABMaterial Dashboard Temperature & Airpressure provided by ESP connected to Raspberry Pi"
	page.PageDescription = "ABMaterial Server Application, running on a Raspberry Pi, to display the Temperature & Pressure of a connected ESP with BMP180 sensor."
	page.PageKeywords = "B4J, ABMaterial, Raspberry Pi, ESP8266, BMP180"
	page.PageSiteMapPriority = ""
	page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY
	page.ShowConnectedIndicator = True
		
	' adding a navigation bar
	
	' create the page grid
	' row 1, 1 cell: header
	page.AddRows(1,False, "headertheme").AddCells12(1,"headertheme")
	' row 2, 4 cells: gauge temperature, chart temperature, gauge pressure, chart pressure
	page.AddRows(1, True,"").AddCellsOS(4,0,0,0,3,3,3,"")
	' row 3, 1 cell: Divider
	page.AddRows(1, True,"").AddCellsOS(2,0,0,0,12,6,6,"")
	' row 4, 1 cell: Information
	page.AddRows(1, True,"").AddCells12(1,"")
	'
	page.BuildGrid 'IMPORTANT loaded the complete grid prior adding components

	' Create controls - IMPORTANT = EACH CONTROL MUST HAVE AN UNIQUE ID
	' Row1,Cell1 = Add label as header
	Dim lblHeader As ABMLabel
	lblHeader.Initialize(page, "lblHeader", "B4J HowTo ABMaterial Dashboard Temperature & Airpressure provided by ESP connected to Raspberry Pi",ABM.SIZE_H4, True, "whitefc")
	lblHeader.VerticalAlign = True
	page.Cell(1,1).AddComponent(lblHeader)

	' Row2,Cell1 = TODO Add Temperature Gauge
	' page.Cell(1,1).AddComponent(GAUGE)
	Dim lblTemperatureGauge As ABMLabel
	lblTemperatureGauge.Initialize(page, "lblTemperatureGauge", "ToDo = Temperature Gauge",ABM.SIZE_H4, True, "whitefc")
	lblTemperatureGauge.VerticalAlign = True
	page.Cell(2,1).AddComponent(lblTemperatureGauge)

	' Row2,Cell2 = Add Temperature Line Chart
	Dim chartT As ABMChart
	chartT.Initialize(page, "chartT", ABM.CHART_TYPELINE, ABM.CHART_RATIO_GOLDENSECTION, "")
	chartT.OptionsLine.FixedHeightPx = 300
	chartT.OptionsLine.AxisYLow=17
	chartT.OptionsLine.AxisYHigh=19
	chartT.OptionsLine.AxisYOnlyInteger=True
	chartT.OptionsLine.FullWidth=True
	chartT.OptionsLine.ChartPaddingRight=60 ' because we set fullwidth, we have to adjust so the final label also fits
	chartT.OptionsLine.Serie(ABM.CHART_SERIEINDEX_A).LineSmooth = ABM.CHART_LINESMOOTH_SIMPLE
	chartT.OptionsLine.Serie(ABM.CHART_SERIEINDEX_A).ShowPoint=False
	CurrentLabelsT.addAll(Array As String("0", "0", "0", "0", "0", "0", "0","0", "0", "0"))
	chartT.AddLabels(CurrentLabelsT)
	CurrentValuesT.AddAll(Array As Int(17,17,17,17,17,17,17,17,17,17))
	Dim SerieTA As ABMChartSerie
	SerieTA.InitializeForLine(ABM.CHART_SERIEINDEX_A)
	SerieTA.SetValues(CurrentValuesT)
	chartT.AddSerie(SerieTA)
	page.Cell(2,2).AddComponent(chartT)

	' Row2,Cell3 = TODO Add Pressure Gauge
	' page.Cell(2,3).AddComponent(GAUGE)
	Dim lblPressureGauge As ABMLabel
	lblPressureGauge.Initialize(page, "lblPressureGauge", "ToDo = Pressure Gauge",ABM.SIZE_H4, True, "whitefc")
	lblPressureGauge.VerticalAlign = True
	page.Cell(2,3).AddComponent(lblPressureGauge)

	' Row2,Cell4 = Add Pressure Line Chart
	Dim chartP As ABMChart
	chartP.Initialize(page, "chartP", ABM.CHART_TYPELINE, ABM.CHART_RATIO_GOLDENSECTION, "")
	chartP.OptionsLine.FixedHeightPx = 300
	chartP.OptionsLine.AxisYLow=1025
	chartP.OptionsLine.AxisYHigh=1030
	chartP.OptionsLine.AxisYOnlyInteger=True
	chartP.OptionsLine.FullWidth=True
	chartP.OptionsLine.ChartPaddingRight=60 ' because we set fullwidth, we have to adjust so the final label also fits
	chartP.OptionsLine.Serie(ABM.CHART_SERIEINDEX_A).LineSmooth = ABM.CHART_LINESMOOTH_SIMPLE
	chartP.OptionsLine.Serie(ABM.CHART_SERIEINDEX_A).ShowPoint=False
	CurrentLabelsP.addAll(Array As String("0", "0", "0", "0", "0", "0", "0","0", "0", "0"))
	chartP.AddLabels(CurrentLabelsP)
	CurrentValuesP.AddAll(Array As Int(0,0,0,0,0,0,0,0,0,0))
	Dim SeriePA As ABMChartSerie
	SeriePA.InitializeForLine(ABM.CHART_SERIEINDEX_A)
	SeriePA.SetValues(CurrentValuesP)
	chartP.AddSerie(SeriePA)
	page.Cell(2,4).AddComponent(chartP)

	' Row 3 = Add hor divider
	Dim div1 As ABMDivider
	div1.Initialize(page, "div1", "lightblue")
	page.Row(3).SetDivider(div1)

	' Row 4 = Empty

	' Fixed Footer
	ABMShared.BuildFooterFixed(page)
End Sub

' clicked on the navigation bar
Sub Page_NavigationbarClicked(Action As String, Value As String)
	page.SaveNavigationBarPosition
End Sub

Sub Page_FileUploaded(FileName As String, success As Boolean)	
	
End Sub

Sub Page_ToastClicked(ToastId As String, Action As String)
		
End Sub

Sub Page_ToastDismissed(ToastId As String)	
	
End Sub

Sub Page_Ready()
	page.RestoreNavigationBarPosition
   	'Dim LEDSwitch As ABMSwitch = page.Component("SwitchLED3")
	'LEDSwitch.State = LEDController.GetPinState(LEDController.Pin3)
   	page.Refresh
	Log("ESPDashboardPage ready!")
End Sub

Sub DataCollectionTimer_Tick
	UpdateCharts(Main.Temperature, Main.Pressure)
End Sub

Public Sub UpdateCharts(t As Double, p As Double)
	'Log($"${DateTime.Time(DateTime.Now)}: DataCollection ${Main.Temperature}, ${Main.Pressure}"$)
	Dim chartT As ABMChart = page.Component("chartT")
	If Main.TemperatureDataReceived Then
	If chartT <> Null Then
		CurrentLabelsT.RemoveAt(0)
		CurrentValuesT.RemoveAt(0)
		CurrentValuesT.Add(t)
		CurrentLabelsT.Add(DateTime.Time(DateTime.Now))
		chartT.SetLabels(CurrentLabelsT)
		chartT.GetSerie(ABM.CHART_SERIEINDEX_A).SetValues(CurrentValuesT)
		chartT.Refresh
		Log($"${DateTime.Time(DateTime.Now)}: ChartT ${t}"$)
	End If
	End If

	Dim chartP As ABMChart = page.Component("chartP")
	If Main.PressureDataReceived Then
	If chartP <> Null Then
		CurrentLabelsP.RemoveAt(0)
		CurrentValuesP.RemoveAt(0)
		CurrentValuesP.Add(p)
		CurrentLabelsP.Add(DateTime.Time(DateTime.Now))
		chartP.SetLabels(CurrentLabelsP)
		chartP.GetSerie(ABM.CHART_SERIEINDEX_A).SetValues(CurrentValuesP)
		chartP.Refresh
		Log($"${DateTime.Time(DateTime.Now)}: ChartP ${p}"$)
	End If
	End If
End Sub
